import ipywidgets as widgets
import bqplot
from traittypes import Array
from bqplot.traits import (array_serialization, array_squeeze)
from traitlets import Int, Unicode, List, Dict, Float, Instance
from bqplot.marks import shape
from bqplot.traits import array_to_json, array_from_json
from bqplot_image_gl._version import __version__

__all__ = ['ImageGL', 'Contour']


@widgets.register
class ImageGL(bqplot.Mark):
    """An example widget."""
    _view_name = Unicode('ImageGLView').tag(sync=True)
    _model_name = Unicode('ImageGLModel').tag(sync=True)
    _view_module = Unicode('bqplot-image-gl').tag(sync=True)
    _model_module = Unicode('bqplot-image-gl').tag(sync=True)
    _view_module_version = Unicode('^' + __version__).tag(sync=True)
    _model_module_version = Unicode('^' + __version__).tag(sync=True)

    image = Array().tag(sync=True,
                        scaled=True,
                        rtype='Color',
                        atype='bqplot.ColorAxis',
                        **array_serialization)
    interpolation = Unicode('nearest', allow_none=True).tag(sync=True)
    opacity = Float(1.0).tag(sync=True)
    x = Array(default_value=(0, 1)).tag(sync=True, scaled=True,
                                        rtype='Number',
                                        atype='bqplot.Axis',
                                        **array_serialization)\
        .valid(array_squeeze, shape(2))
    y = Array(default_value=(0, 1)).tag(sync=True, scaled=True,
                                        rtype='Number',
                                        atype='bqplot.Axis',
                                        **array_serialization)\
        .valid(array_squeeze, shape(2))
    scales_metadata = Dict({
        'x': {'orientation': 'horizontal', 'dimension': 'x'},
        'y': {'orientation': 'vertical', 'dimension': 'y'},
        'image': {'dimension': 'color'},
    }).tag(sync=True)


def double_list_array_from_json(double_list):
    return [[array_from_json(k) for k in array_list] for array_list in double_list]


def double_list_array_to_json(double_list, obj=None):
    return [[array_to_json(k) for k in array_list] for array_list in double_list]


double_list_array_serialization = dict(to_json=double_list_array_to_json,
                                       from_json=double_list_array_from_json)


@widgets.register
class Contour(bqplot.Mark):
    _view_name = Unicode('ContourView').tag(sync=True)
    _model_name = Unicode('ContourModel').tag(sync=True)
    _view_module = Unicode('bqplot-image-gl').tag(sync=True)
    _model_module = Unicode('bqplot-image-gl').tag(sync=True)
    _view_module_version = Unicode('^' + __version__).tag(sync=True)
    _model_module_version = Unicode('^' + __version__).tag(sync=True)

    image = Instance(ImageGL, allow_none=True).tag(sync=True, **widgets.widget_serialization)
    label_steps = Int(40).tag(sync=True)
    contour_lines = (List(List(Array(None, allow_none=True)))
                     .tag(sync=True, **double_list_array_serialization))
    level = (Float() | List(Float())).tag(sync=True)
    label = (Unicode(None, allow_none=True) | List(Unicode())).tag(sync=True)
    color = (widgets.Color(None, allow_none=True) |
             List(widgets.Color(None, allow_none=True))).tag(sync=True)
    scales_metadata = Dict({
        'x': {'orientation': 'horizontal', 'dimension': 'x'},
        'y': {'orientation': 'vertical', 'dimension': 'y'},
    }).tag(sync=True)



import ipywidgets as widgets
import bqplot
from traitlets import Unicode
from bqplot_image_gl._version import __version__

__all__ = ['LinesGL']


@widgets.register
class LinesGL(bqplot.Lines):
    """An example widget."""
    _view_name = Unicode('LinesGLView').tag(sync=True)
    _model_name = Unicode('LinesGLModel').tag(sync=True)
    _view_module = Unicode('bqplot-image-gl').tag(sync=True)
    _model_module = Unicode('bqplot-image-gl').tag(sync=True)
    _view_module_version = Unicode('^' + __version__).tag(sync=True)
    _model_module_version = Unicode('^' + __version__).tag(sync=True)



from bqplot.interacts import BrushSelector, Interaction
from bqplot.scales import Scale
from traitlets import Float, Unicode, Dict, Instance, Int, List
from ipywidgets.widgets.widget import widget_serialization
from bqplot_image_gl._version import __version__


drag_events = ["dragstart", "dragmove", "dragend"]
mouse_events = ['click', 'dblclick', 'mouseenter', 'mouseleave', 'contextmenu', 'mousemove']
keyboard_events = ['keydown', 'keyup']


class BrushEllipseSelector(BrushSelector):

    """BrushEllipse interval selector interaction.

    This 2-D selector interaction enables the user to select an ellipse
    region using the brushing action of the mouse. A mouse-down marks the
    center of the ellipse. The drag after the mouse down selects a point
    on the ellipse, drawn with the same aspect ratio as the change in x and y
    as measured in pixels. If pixel_aspect is set, the aspect ratio of the ellipse
    will be used instead. Note that the aspect ratio is respected in the view
    where the ellipse is drawn.

    Once an ellipse is drawn, it can be moved dragging, or reshaped by dragging
    the border.

    The selected_x and selected_y arrays define the bounding box of the ellipse.

    Attributes
    ----------
    selected_x: numpy.ndarray
        Two element array containing the start and end of the interval selected
        in terms of the x_scale of the selector.
        This attribute changes while the selection is being made with the
        ``BrushSelector``.
    selected_y: numpy.ndarray
        Two element array containing the start and end of the interval selected
        in terms of the y_scale of the selector.
        This attribute changes while the selection is being made with the
        ``BrushEllipseSelector``.
    brushing: bool (default: False)
        boolean attribute to indicate if the selector is being dragged.
        It is True when the selector is being moved and False when it is not.
        This attribute can be used to trigger computationally intensive code
        which should be run only on the interval selection being completed as
        opposed to code which should be run whenever selected is changing.
    """
    _view_module = Unicode('bqplot-image-gl').tag(sync=True)
    _model_module = Unicode('bqplot-image-gl').tag(sync=True)
    _view_module_version = Unicode('^' + __version__).tag(sync=True)
    _model_module_version = Unicode('^' + __version__).tag(sync=True)
    pixel_aspect = Float(None, allow_none=True).tag(sync=True)
    style = Dict({"opacity": 0.3, "cursor": "grab"}).tag(sync=True)
    border_style = Dict({"fill": "none", "stroke-width": "3px",
                         "opacity": 0.3, "cursor": "col-resize"}).tag(sync=True)
    _view_name = Unicode('BrushEllipseSelector').tag(sync=True)
    _model_name = Unicode('BrushEllipseSelectorModel').tag(sync=True)


class MouseInteraction(Interaction):
    """Mouse events listener.

    Listen for mouse events on the kernel side.
    The attributes 'x_scale' and 'y_scale' should be provided.

    Event being passed are
        * dragstart
        * dragmove
        * dragend
        * click
        * dblclick

    All events are passed by a custom event with the following spec:
    `{event: <name>, pixel: {x: <pixel x>, y: <pixel y>}, domain: {x: <x>, y: <y>}} `

    Pixel coordinates might be useful for debugging, domain coordinates should be used only.

    Attributes
    ----------
    x_scale: An instance of Scale
        This is the scale which is used for inversion from the pixels to data
        co-ordinates in the x-direction.
    y_scale: An instance of Scale
        This is the scale which is used for inversion from the pixels to data
        co-ordinates in the y-direction.
    move_throttle: Send mouse move events only every specified milliseconds.
    """
    _view_module = Unicode('bqplot-image-gl').tag(sync=True)
    _model_module = Unicode('bqplot-image-gl').tag(sync=True)
    _view_module_version = Unicode('^' + __version__).tag(sync=True)
    _model_module_version = Unicode('^' + __version__).tag(sync=True)
    _view_name = Unicode('MouseInteraction').tag(sync=True)
    _model_name = Unicode('MouseInteractionModel').tag(sync=True)
    x_scale = Instance(Scale, allow_none=True, default_value=None)\
        .tag(sync=True, dimension='x', **widget_serialization)
    y_scale = Instance(Scale, allow_none=True, default_value=None)\
        .tag(sync=True, dimension='y', **widget_serialization)
    cursor = Unicode('auto').tag(sync=True)
    move_throttle = Int(50).tag(sync=True)
    next = Instance(Interaction, allow_none=True).tag(sync=True, **widget_serialization)
    events = List(Unicode, default_value=drag_events + mouse_events + keyboard_events,
                  allow_none=True).tag(sync=True)



from ._version import version_info, __version__  # noqa

from .imagegl import *  # noqa
from .linesgl import *  # noqa


def _prefix():
    import sys
    from pathlib import Path
    prefix = sys.prefix
    here = Path(__file__).parent
    # for when in dev mode
    if (here.parent / 'share/jupyter/nbextensions/bqplot-image-gl').exists():
        prefix = here.parent
    return prefix


def _jupyter_labextension_paths():
    return [{
        'src': f'{_prefix()}/share/jupyter/labextensions/bqplot-image-gl/',
        'dest': 'bqplot-image-gl',
    }]


def _jupyter_nbextension_paths():
    return [{
        'section': 'notebook',
        'src': f'{_prefix()}/share/jupyter/nbextensions/bqplot-image-gl/',
        'dest': 'bqplot-image-gl',
        'require': 'bqplot-image-gl/extension'
    }]



version_info = (1, 4, 5, 'final', 0)

_specifier_ = {'alpha': 'a', 'beta': 'b', 'candidate': 'rc', 'final': ''}

__version__ = '%s.%s.%s%s' % (version_info[0],
                              version_info[1],
                              version_info[2],
                              '' if version_info[3] == 'final'
                              else _specifier_[version_info[3]] + str(version_info[4]))



import numpy as np
import bqplot
from bqplot_image_gl import ImageGL


def test_astro_image():
    data = np.zeros((2, 3))
    color_scale = bqplot.ColorScale()
    scale_x = bqplot.LinearScale()
    scale_y = bqplot.LinearScale()
    ImageGL(image=data, scales={'image': color_scale, 'x': scale_x, 'y': scale_y})



from __future__ import print_function
from os.path import join as pjoin
from setuptools import setup, find_packages, Command
import os

from jupyter_packaging import (
    create_cmdclass,
    install_npm,
    ensure_targets,
    combine_commands,
    get_version,
    skip_if_exists,
)

LONG_DESCRIPTION = 'An ipywidget image widget for astronomical purposes'
here = os.path.dirname(os.path.abspath(__file__))
name = 'bqplot-image-gl'
package_name = name.replace('-', '_')
version = get_version(pjoin(package_name, '_version.py'))

js_dir = pjoin(here, 'js')

# Representative files that should exist after a successful build
jstargets = [
    pjoin('share', 'jupyter', 'nbextensions', f'{name}', 'index.js'),
    # pjoin('share', 'jupyter', 'labextensions', f'{name}', 'package.json'),
]

data_files_spec = [
    (f'share/jupyter/nbextensions/{name}', f'share/jupyter/nbextensions/{name}', '*.js'),
    (f'share/jupyter/labextensions/{name}/', f'share/jupyter/labextensions/{name}/', '**'),
    (f'etc/jupyter/nbconfig/notebook.d', f'etc/jupyter/nbconfig/notebook.d', f'{name}.json'),
]

js_command = combine_commands(
    install_npm(js_dir, build_dir='share/jupyter/', source_dir='js/src', build_cmd='build'), ensure_targets(jstargets),
)

cmdclass = create_cmdclass('jsdeps', data_files_spec=data_files_spec)
is_repo = os.path.exists(os.path.join(here, '.git'))
if is_repo:
    cmdclass['jsdeps'] = js_command
else:
    cmdclass['jsdeps'] = skip_if_exists(jstargets, js_command)

setup(
    name=name,
    version=version,
    description='An ipywidget image widget for astronomical purposes',
    long_description=LONG_DESCRIPTION,
    include_package_data=True,
    install_requires=[
        'ipywidgets>=7.0.0',
        'bqplot>=0.12'
    ],
    packages=find_packages(),
    zip_safe=False,
    cmdclass=cmdclass,
    author='Maarten A. Breddels',
    author_email='maartenbreddels@gmail.com',
    url='https://github.com/glue-viz/bqplot-image-gl',
    keywords=[
        'ipython',
        'jupyter',
        'widgets',
    ],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Framework :: IPython',
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        'Topic :: Multimedia :: Graphics',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
)



import ipywidgets as widgets
from ipywidgets.widgets import widget_serialization
from traitlets import Unicode, Dict, Instance
from bqplot_image_gl._version import __version__

__all__ = ['ViewListener']


@widgets.register
class ViewListener(widgets.DOMWidget):
    _view_name = Unicode('ViewListener').tag(sync=True)
    _model_name = Unicode('ViewListenerModel').tag(sync=True)
    _view_module = Unicode('bqplot-image-gl').tag(sync=True)
    _model_module = Unicode('bqplot-image-gl').tag(sync=True)
    _view_module_version = Unicode('^' + __version__).tag(sync=True)
    _model_module_version = Unicode('^' + __version__).tag(sync=True)

    widget = Instance(widgets.Widget).tag(sync=True, **widget_serialization)
    css_selector = Unicode(None, allow_none=True).tag(sync=True)
    view_data = Dict().tag(sync=True)
